﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using Telerik.WinControls;
using Telerik.WinControls.UI;

namespace RadMultiSelectDropDownListImpl
{
    public class CustomListDataItem : RadListDataItem
    {
        public event ChangedEventHandler Changed;

        #region Contructors

        public CustomListDataItem()
        {
            this.PropertyChanged += new PropertyChangedEventHandler(item_PropertyChanged);
        }

        public CustomListDataItem(string text)
            : base(text)
        {
            this.PropertyChanged += new PropertyChangedEventHandler(item_PropertyChanged);
        }

        #endregion

        #region RadProperties

        public static readonly RadProperty CheckedProperty = RadProperty.Register("Checked", typeof(bool), typeof(CustomListDataItem), new RadElementPropertyMetadata(false));
        public static readonly RadProperty NameProperty = RadProperty.Register("Name", typeof(string), typeof(CustomListDataItem), new RadElementPropertyMetadata(""));

        #endregion

        #region Properties

        public bool Checked
        {
            get
            {
                return (bool)this.GetValue(CustomListDataItem.CheckedProperty);
            }
            set
            {
                this.SetValue(CustomListDataItem.CheckedProperty, value);
            }
        }

        public string Name
        {
            get
            {
                return (string)this.GetValue(CustomListDataItem.NameProperty);
            }
            set
            {
                this.SetValue(CustomListDataItem.NameProperty, value);
            }
        }

        #endregion

        protected virtual void OnSelectedItemChanged(ItemsChangedEventArgs e)
        {
            if (Changed != null)
            {
                Changed(this, e);
            }
        }

        #region Private Methods

        private void item_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Checked")
            {
                RadMultiSelectDropDownList ddlControl = (RadMultiSelectDropDownList)((DropDownPopupForm)this.Owner.ElementTree.Control).OwnerDropDownListElement.ElementTree.Control;
                ddlControl.SetText();

                if (this.Checked)
                {
                    OnSelectedItemChanged(new ItemsChangedEventArgs(this, ItemsChangeOperation.Inserted));
                }
                else
                {
                    OnSelectedItemChanged(new ItemsChangedEventArgs(this, ItemsChangeOperation.Removed));
                }
            }
        }

        #endregion
    }
}
