﻿using System;
using Telerik.WinControls.UI;
using Telerik.WinControls;
using Telerik.WinControls.Layouts;
using System.Windows.Forms;

namespace RadMultiSelectDropDownListImpl
{
    public class CustomListVisualItem : RadListVisualItem
    {
        #region Fields

        private RadLabelElement label = new RadLabelElement();
        private RadCheckBox checkBox = new RadCheckBox();
        private RadHostItem host;

        #endregion

        #region Initialization

        static CustomListVisualItem()
        {
            RadListVisualItem.SynchronizationProperties.Add(CustomListDataItem.CheckedProperty);
            RadListVisualItem.SynchronizationProperties.Add(CustomListDataItem.NameProperty);
        }

        #endregion

        #region Overrides

        protected override Type ThemeEffectiveType
        {
            get
            {
                return typeof(RadListVisualItem);
            }
        }

        protected override void CreateChildElements()
        {
            base.CreateChildElements();

            this.host = new RadHostItem(checkBox);
            this.checkBox.ToggleStateChanged += this.ToggleStateChanged;
            this.label.StretchHorizontally = true;
            this.label.Margin = new System.Windows.Forms.Padding(15, 0, 0, 0);
            this.host.Margin = new System.Windows.Forms.Padding(0, 1, 0, 0);
            StackLayoutPanel stack = new StackLayoutPanel();
            stack.Orientation = Orientation.Horizontal;
            stack.Children.Add(host);
            stack.Children.Add(label);
            this.Children.Add(stack);
        }

        protected override void PropertySynchronized(RadProperty property)
        {
            CustomListDataItem dataItem = (CustomListDataItem)this.Data;

            if (property == CustomListDataItem.CheckedProperty)
            {
                bool newValue = dataItem.Checked;

                this.checkBox.Checked = newValue;

                string itemName = dataItem.Text;
                this.label.Text = this.GetAvailabilityText(itemName, newValue);
            }

            if (property == CustomListDataItem.NameProperty)
            {
                string itemName = dataItem.Text;
                bool available = dataItem.Checked;
                this.label.Text = this.GetAvailabilityText(itemName, available);
            }

            this.Text = "";
            RadMultiSelectDropDownList ddlControl = (RadMultiSelectDropDownList)((DropDownPopupForm)this.ElementTree.Control).OwnerDropDownListElement.ElementTree.Control;
            ddlControl.SetText();

        }

        #endregion

        #region Private Methods

        private string GetAvailabilityText(string itemName, bool available)
        {
            string yesno = available ? " " : " not ";
            return itemName + " is" + yesno + "checked";
        }

        private void ToggleStateChanged(object sender, StateChangedEventArgs args)
        {
            ((CustomListDataItem)this.Data).Checked = this.checkBox.Checked;
        }

        #endregion
    }
}
