﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.WinControls.UI;
using Telerik.WinControls;
using System.ComponentModel;
using Telerik.WinControls.Data;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace RadMultiSelectDropDownListImpl
{
    public delegate void ChangedEventHandler(object sender, ItemsChangedEventArgs e);

    public class RadMultiSelectDropDownList : RadDropDownList
    {
        [DllImport("user32")]
        protected static extern bool HideCaret(IntPtr hWnd); 

        MyDynamicCollection selectedItems;
        public event ChangedEventHandler SelectedItemsChanged;

        public RadMultiSelectDropDownList()
        {
            selectedItems = new MyDynamicCollection(this);

            WireEvents();
        }

        private void WireEvents()
        {
            this.ListElement.ItemsChanged += new NotifyCollectionChangedEventHandler(ListElement_ItemsChanged);
            this.ListElement.ItemsChanging += new NotifyCollectionChangingEventHandler(ListElement_ItemsChanging);

            this.DropDownListElement.TextBox.TextBoxItem.HostedControl.GotFocus += new EventHandler(HostedControl_GotFocus);
        }

        void HostedControl_GotFocus(object sender, EventArgs e)
        {
            HideCaret(((TextBox)this.DropDownListElement.TextBox.TextBoxItem.HostedControl).Handle);
        }

        void ListElement_ItemsChanging(object sender, NotifyCollectionChangingEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Reset)
            {
                for (int i = 0; i < this.Items.Count; i++)
                {
                    ((CustomListDataItem)this.Items[i]).Changed -= new ChangedEventHandler(OnSelectedItemChanged);
                }
            }
        }

        void ListElement_ItemsChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                ((CustomListDataItem)e.NewItems[0]).Changed += new ChangedEventHandler(OnSelectedItemChanged);
            }

            if (e.Action == NotifyCollectionChangedAction.Remove)
            {
                ((CustomListDataItem)e.OldItems[0]).Changed -= new ChangedEventHandler(OnSelectedItemChanged);
            }
        }

        protected virtual void OnSelectedItemChanged(object sender, ItemsChangedEventArgs e)
        {
            if (this.SelectedItemsChanged != null)
            {
                SelectedItemsChanged(this, e);
            }
        }

        protected override void Dispose(bool disposing)
        {
            this.UnwireEvents();
            base.Dispose(disposing);
        }

        private void UnwireEvents()
        {
            this.ListElement.ItemsChanged -= new NotifyCollectionChangedEventHandler(ListElement_ItemsChanged);
            this.ListElement.ItemsChanging -= new NotifyCollectionChangingEventHandler(ListElement_ItemsChanging);

            this.DropDownListElement.TextBox.TextBoxItem.HostedControl.GotFocus -= new EventHandler(HostedControl_GotFocus);
        }

        public MyDynamicCollection SelectedItems
        {
            get
            {
                return selectedItems;
            }
        }

        protected override void OnCreatingVisualItem(object sender, CreatingVisualListItemEventArgs args)
        {
            base.OnCreatingVisualItem(sender, args);

            args.VisualItem = new CustomListVisualItem();
        }

        public override string ThemeClassName
        {
            get
            {
                return typeof(RadDropDownList).FullName;
            }
        }

        protected override void OnSelectedIndexChanged(object sender, int newIndex)
        {
            base.OnSelectedIndexChanged(sender, newIndex);

            SetText();
        }

        internal void SetText()
        {
            this.BeginUpdate();
            this.Text = String.Empty;
            foreach (CustomListDataItem item in this.Items)
            {
                if (item.Checked)
                {
                    this.DropDownListElement.EditableElementText += item.Text + "; ";
                }
            }
            this.EndUpdate();
        }
    }
}
